
import { Schemas } from '../../../../base/common/network.js';
import { URI } from '../../../../base/common/uri.js';
import { stringifyPromptElementJSON } from './tools/promptTsxTypes.js';

var ToolDataSource;
(function (ToolDataSource) {
    function toKey(source) {
        switch (source.type) {
            case 'extension': return `extension:${source.extensionId.value}`;
            case 'mcp': return `mcp:${source.collectionId}:${source.definitionId}`;
            case 'internal': return 'internal';
        }
    }
    ToolDataSource.toKey = toKey;
})(ToolDataSource || (ToolDataSource = {}));
function isToolInvocationContext(obj) {
    return typeof obj === 'object' && typeof obj.sessionId === 'string';
}
function isToolResultInputOutputDetails(obj) {
    return typeof obj === 'object' && typeof obj?.input === 'string' && (typeof obj?.output === 'string' || Array.isArray(obj?.output));
}
function toolResultHasBuffers(result) {
    return ( result.content.some(part => part.kind === 'data'));
}
function stringifyPromptTsxPart(part) {
    return stringifyPromptElementJSON(part.value);
}
function createToolInputUri(toolOrId) {
    if (typeof toolOrId !== 'string') {
        toolOrId = toolOrId.id;
    }
    return ( URI.from({ scheme: Schemas.inMemory, path: `/lm/tool/${toolOrId}/tool_input.json` }));
}
function createToolSchemaUri(toolOrId) {
    if (typeof toolOrId !== 'string') {
        toolOrId = toolOrId.id;
    }
    return ( URI.from(
        { scheme: Schemas.vscode, authority: 'schemas', path: `/lm/tool/${toolOrId}` }
    ));
}

export { ToolDataSource, createToolInputUri, createToolSchemaUri, isToolInvocationContext, isToolResultInputOutputDetails, stringifyPromptTsxPart, toolResultHasBuffers };
